==> haxt-0.3.62.tgz: package/source/Components/Fragment.tsx <==
import { Component, TSX, h } from "../TSX/TSX";
import { BasicComponent } from "./BasicComponent";
import { UniqueArray } from "@h4x/common";

export abstract class Fragment<T> extends BasicComponent<T & { fragmentList: Fragment.List<Fragment<T>> }> {
	// TODO prevent direct use, this should only be used with TSX.prerenderComponent
	private rendered: TSX.RenderedComponent<Fragment<T>>;

	public remove() {
		this.props.fragmentList.removeFragment(this.rendered);
		this.rendered.destroy();
	}

	public moveTo(pos: number) {
		this.props.fragmentList.moveFragment(this.rendered, pos);
	}

	public moveUp() {
		let index = this.props.fragmentList.indexOf(this.rendered);
		this.props.fragmentList.moveFragment(this.rendered, index - 1);
	}

	public moveDown() {
		let index = this.props.fragmentList.indexOf(this.rendered);
		// +2 as it need to move over itself and the next elements
		this.props.fragmentList.moveFragment(this.rendered, index + 2);
	}

	public override onDetached() {
		console.log("onDetached");
	}

	public static attachRendered<K extends Fragment<any>>(instance: K, rendered: TSX.RenderedComponent<K>) {
		instance.rendered = rendered;
	}
}

export namespace Fragment {
	export class List<T extends Fragment<any>> {
		private fragments = new UniqueArray<TSX.RenderedComponent<T>>();
		private component: Container<T>;

		public get count() { return this.fragments.length; }

		public addFragment(FragmentClass: new () => T, args: Omit<TSX.Props<T>, "fragmentList">, at?: number) {
			let fragment = <FragmentClass {...args} fragmentList={this} />;
			let rendered = TSX.prerenderComponent<T>(fragment);
			let instance = rendered.vnode.instance;
			Fragment.attachRendered(instance, rendered);

			if (at !== undefined) {
				this.fragments.insertAt(at, rendered);
			} else if (at === 0) {
				this.fragments.unshift(rendered);
			} else {
				this.fragments.push(rendered);
			}

			if (this.component !== undefined) {
				this.component.attachFragment(rendered, at);
			}
			return instance;
		}

		public moveFragment(rendered: TSX.RenderedComponent<T>, to?: number) {
			let instance = rendered.vnode.instance;

			if (to !== undefined) {
				this.fragments.insertAt(to, rendered);
			} else if (to === 0) {
				this.fragments.unshift(rendered);
			} else {
				this.fragments.push(rendered);
			}

			if (this.component !== undefined) {
				this.component.attachFragment(rendered, to);
			}
			return instance;
		}

		// swap? swap range? move range?

		public removeFragment(rendered: TSX.RenderedComponent<T>) {
			if (this.fragments.removeElement(rendered)) {
				return true;
			}
			return false;
		}

		public indexOf(rendered: TSX.RenderedComponent<T>) {
			return this.fragments.indexOf(rendered);
		}

		static attachComponent<T extends Fragment<any>>(list: List<T>, component: Container<T>) {
			if (list.component !== undefined) { throw new Error("FragmentList already attached to a component"); }
			list.component = component;

			for (const fragment of list.fragments) {
				list.component.attachFragment(fragment);
			}
		}
	}

	export namespace Container {
		export interface Props<T extends Fragment<any>> extends JSX.HTMLAttributes<HTMLDivElement> {
			fragmentList: List<T>;
			[Component.Children]?: undefined;
		}
	}
	export class Container<T extends Fragment<any>> extends BasicComponent<Container.Props<T>>{
		private container: HTMLDivElement;
		@BasicComponent.bind()
		public attachContainer(instance: HTMLDivElement) {
			if (this.container !== undefined) {
				if (this.container !== instance) {
					throw new Error("Trying to reassign fragment container!");
				}
			} else {
				this.container = instance;
				List.attachComponent(this.props.fragmentList, this);
			}
		}

		attachFragment(rendered: TSX.RenderedComponent<T>, at?: number | Node) {
			rendered.attach(this.container, at);
		}

		public render() {
			const { fragmentList, ...props } = this.props;
			return <raw {...props} key="fragments" ref={this.attachContainer} />;
		}
	}
}

==> haxt-0.3.63.tgz: package/source/Components/Fragment.tsx <==
import { Component, TSX, h } from "../TSX/TSX";
import { BasicComponent } from "./BasicComponent";
import { UniqueArray } from "@h4x/common";

export abstract class Fragment<T> extends BasicComponent<T & { fragmentList: Fragment.List<Fragment<T>> }> {
	// TODO prevent direct use, this should only be used with TSX.prerenderComponent
	private rendered: TSX.RenderedComponent<Fragment<T>>;

	public remove() {
		this.props.fragmentList.removeFragment(this.rendered);
		this.rendered.destroy();
	}

	public moveTo(pos: number) {
		this.props.fragmentList.moveFragment(this.rendered, pos);
	}

	public moveUp() {
		let index = this.props.fragmentList.indexOf(this.rendered);
		this.props.fragmentList.moveFragment(this.rendered, index - 1);
	}

	public moveDown() {
		let index = this.props.fragmentList.indexOf(this.rendered);
		// +2 as it need to move over itself and the next elements
		this.props.fragmentList.moveFragment(this.rendered, index + 2);
	}

	public override onDetached() {
		console.log("Fragment onDetached");
	}

	public static attachRendered<K extends Fragment<any>>(instance: K, rendered: TSX.RenderedComponent<K>) {
		instance.rendered = rendered;
	}
}

export namespace Fragment {
	export class List<T extends Fragment<any>> {
		private readonly fragments = new UniqueArray<TSX.RenderedComponent<T>>();
		private component: Container<T>;

		public get length() { return this.fragments.length; }
		public get first() { return this.fragments[0]; }
		public get firstInstance() { return this.fragments[0]?.vnode.instance; }
		public get last() { return this.fragments[this.fragments.length - 1]; }
		public get lastInstance() { return this.fragments[this.fragments.length - 1]?.vnode.instance; }

		public *[Symbol.iterator]() {
			for (const fragment of this.fragments) {
				yield fragment;
			}
		}

		public *instances() {
			for (const fragment of this.fragments) {
				yield fragment.vnode.instance;
			}
		}

		public get(at: number) {
			return this.fragments[at];
		}

		public getInstance(at: number) {
			return this.fragments[at].vnode.instance;
		}

		public addFragment(FragmentClass: new () => T, args: Omit<TSX.Props<T>, "fragmentList">, at?: number) {
			let fragment = <FragmentClass {...args} fragmentList={this} />;
			let rendered = TSX.prerenderComponent<T>(fragment);
			let instance = rendered.vnode.instance;
			Fragment.attachRendered(instance, rendered);

			if (at !== undefined) {
				this.fragments.insertAt(at, rendered);
			} else if (at === 0) {
				this.fragments.unshift(rendered);
			} else {
				this.fragments.push(rendered);
			}

			if (this.component !== undefined) {
				this.component.attachFragment(rendered, at);
			}
			return instance;
		}

		public moveFragment(rendered: TSX.RenderedComponent<T>, to?: number) {
			let instance = rendered.vnode.instance;

			if (to !== undefined) {
				this.fragments.insertAt(to, rendered);
			} else if (to === 0) {
				this.fragments.unshift(rendered);
			} else {
				this.fragments.push(rendered);
			}

			if (this.component !== undefined) {
				this.component.attachFragment(rendered, to);
			}
			return instance;
		}


		/*
			FragmentList needs to have a way to move Fragments around (and to sort them?)
			swap? swap range? move range?
		*/

		/** @internal */
		public removeFragment(rendered: TSX.RenderedComponent<T>) {
			if (this.fragments.removeElement(rendered)) {
				return true;
			}
			return false;
		}

		public detachFragment(rendered: TSX.RenderedComponent<T>) {
			if (this.fragments.removeElement(rendered)) {
				rendered.detach();
				return true;
			}
			return false;
		}

		public indexOf(rendered: TSX.RenderedComponent<T>) {
			return this.fragments.indexOf(rendered);
		}

		public detachAll() {
			for (const fragment of this.fragments) {
				fragment.detach();
			}
			this.fragments.clear();
		}

		public destroyAll() {
			for (const fragment of this.fragments) {
				fragment.destroy();
			}
			this.fragments.clear();
		}

		static attachComponent<T extends Fragment<any>>(list: List<T>, component: Container<T>) {
			if (list.component !== undefined) { throw new Error("FragmentList already attached to a component"); }
			list.component = component;

			for (const fragment of list.fragments) {
				list.component.attachFragment(fragment);
			}
		}
	}

	export namespace Container {
		export interface Props<T extends Fragment<any>> extends JSX.HTMLAttributes<HTMLDivElement> {
			fragmentList: List<T>;
			[Component.Children]?: undefined;
		}
	}
	export class Container<T extends Fragment<any>> extends BasicComponent<Container.Props<T>>{
		private container: HTMLDivElement;
		@BasicComponent.bind()
		public attachContainer(instance: HTMLDivElement) {
			if (this.container !== undefined) {
				if (this.container !== instance) {
					throw new Error("Trying to reassign fragment container!");
				}
			} else {
				this.container = instance;
				List.attachComponent(this.props.fragmentList, this);
			}
		}

		attachFragment(rendered: TSX.RenderedComponent<T>, at?: number | Node) {
			rendered.attach(this.container, at);
		}

		public render() {
			const { fragmentList, ...props } = this.props;
			return <raw {...props} key="fragments" ref={this.attachContainer} />;
		}
	}
}
